"""Load the config for JARVIS"""

from enum import Enum
from typing import Optional

from pydantic_settings import BaseSettings, SettingsConfigDict


class Mongo(BaseSettings):
    """MongoDB config."""

    host: list[str] | str = "localhost"
    username: Optional[str] = None
    password: Optional[str] = None
    port: int = 27017


class Redis(BaseSettings):
    """Redis config."""

    host: str = "localhost"
    username: Optional[str] = None
    password: Optional[str] = None


class Mastodon(BaseSettings):
    """Mastodon config."""

    token: str
    url: str


class Environment(Enum):
    """JARVIS running environment."""

    production = "production"
    develop = "develop"


class Config(BaseSettings, case_sensitive=False):
    """JARVIS config model."""

    token: str
    """Bot token"""
    erapi: str
    """exchangerate-api.org API token"""
    environment: Environment = Environment.develop
    mongo: Mongo = Mongo()
    redis: Redis = Redis()
    sync: bool = False
    log_level: str = "INFO"
    jurigged: bool = False

    model_config = SettingsConfigDict(env_file=".env", env_file_encoding="utf-8", env_nested_delimiter=".")


def load_config() -> Config:
    """Load the config."""
    return Config()
